@@ -58,18 +58,23 @@ function(xdata.s1, ydata.s1 = NULL, zdata, xdata.s2 = NULL, ydata.s2,
         dimnames(lp.ndea)[[2]] <- temp
         
         # Objective
-        if(orientation == "o") stop('DJL is too busy to implement OO model.')
+        if(orientation == "o") set.objfn(lp.ndea,  c(xdata.s1[k,], 1), 
+                                         indices = c(id.v.s1, id.w.s1))
         if(orientation == "i") set.objfn(lp.ndea,  c(if(is.null(ydata.s1)) NULL else -ydata.s1[k,], -zdata[k,], 1), 
                                          indices = c(if(is.null(ydata.s1)) NULL else id.u.s1, id.p, id.w.s1))
         
-        # RTS
+        # CRS
         if(rts == "crs") add.constraint(lp.ndea, c(1, 1), indices = c(id.w.s1, id.w.s2), "=", 0)
-        if(rts == "irs") stop('DJL is too busy to implement IRS model.')
-        if(rts == "drs") stop('DJL is too busy to implement DRS model.')
-        
+
         # Constraint for o
-        add.constraint(lp.ndea, xdata.s1[k,], indices = id.v.s1, "=", 1)
-        
+        if(orientation == "o"){
+          add.constraint(lp.ndea, c(zdata[k,], if(is.null(ydata.s1)) NULL else ydata.s1[k,]), 
+                         indices = c(id.p, if(is.null(ydata.s1)) NULL else id.u.s1), "=", 1)
+        }
+        if(orientation == "i"){
+          add.constraint(lp.ndea, xdata.s1[k,], indices = id.v.s1, "=", 1)  
+        }
+
         # Constraint for all
         for(d in o){
           # Stage 1
@@ -89,16 +94,18 @@ function(xdata.s1, ydata.s1 = NULL, zdata, xdata.s2 = NULL, ydata.s2,
                                      id.u.s2, id.w.s2), "<=", 0)  
         }
         
-        # Bounds
-        temp.lb <- rep(0, no.dv.t)
+        # Bounds for VRS/IRS
+        temp.lb <- rep(  0, no.dv.t)
+        temp.ub <- rep(Inf, no.dv.t)
         if(rts == "vrs"){temp.lb[c(id.w.s1, id.w.s2)] <- -Inf}
-        set.bounds(lp.ndea, lower = temp.lb)  
+        if(rts == "irs"){temp.lb[c(id.w.s1, id.w.s2)] <- -Inf; temp.ub[c(id.w.s1, id.w.s2)] <- 0}
+        set.bounds(lp.ndea, lower = temp.lb, upper = temp.ub)  
         
         # Solve
         solve.lpExtPtr(lp.ndea)
         
         # Get maximum possible results of Stage 1
-        res.eff.max <- abs(get.objective(lp.ndea))
+        res.eff.max <- ifelse(orientation == "i", abs(get.objective(lp.ndea)), 1/abs(get.objective(lp.ndea)))
         
         # Heuristic search
         res.eff.cand <- c()
@@ -127,19 +134,20 @@ function(xdata.s1, ydata.s1 = NULL, zdata, xdata.s2 = NULL, ydata.s2,
           dimnames(lp.ndea)[[2]] <- temp
           
           # Objective
-          if(orientation == "o") stop('DJL is too busy to implement OO model.')
+          if(orientation == "o") set.objfn(lp.ndea,  c(zdata[k,], if(is.null(xdata.s2)) NULL else xdata.s2[k,], 1), 
+                                           indices = c(id.p, if(is.null(xdata.s2)) NULL else id.v.s2, id.w.s2))
           if(orientation == "i") set.objfn(lp.ndea,  c(-ydata.s2[k,] * res.eff.temp, 1 * res.eff.temp), 
                                            indices = c(id.u.s2, id.w.s2))
           
-          # RTS
+          # CRS
           if(rts == "crs") add.constraint(lp.ndea, c(1, 1), indices = c(id.w.s1, id.w.s2), "=", 0)
-          if(rts == "irs") stop('DJL is too busy to implement IRS model.')
-          if(rts == "drs") stop('DJL is too busy to implement DRS model.')
-          
+
           # Constraint for o
-          add.constraint(lp.ndea, c(zdata[k,], if(is.null(xdata.s2)) NULL else xdata.s2[k,]), 
-                         indices = c(id.p, if(is.null(xdata.s2)) NULL else id.v.s2), "=", 1)
-          
+          if(orientation == "o") add.constraint(lp.ndea, c(ydata.s2[k,] * res.eff.temp), 
+                                                indices = c(id.u.s2), "=", 1)
+          if(orientation == "i") add.constraint(lp.ndea, c(zdata[k,], if(is.null(xdata.s2)) NULL else xdata.s2[k,]), 
+                                                indices = c(id.p, if(is.null(xdata.s2)) NULL else id.v.s2), "=", 1)  
+
           # Eff linking constraint
           add.constraint(lp.ndea, c(-xdata.s1[k,] * res.eff.temp, 
                                     if(is.null(ydata.s1)) NULL else ydata.s1[k,], zdata[k,], -1), 
@@ -164,16 +172,18 @@ function(xdata.s1, ydata.s1 = NULL, zdata, xdata.s2 = NULL, ydata.s2,
                                        id.u.s2, id.w.s2), "<=", 0)  
           }
           
-          # Bounds
-          temp.lb <- rep(0, no.dv.t)
+          # Bounds for VRS/IRS
+          temp.lb <- rep(  0, no.dv.t)
+          temp.ub <- rep(Inf, no.dv.t)
           if(rts == "vrs"){temp.lb[c(id.w.s1, id.w.s2)] <- -Inf}
-          set.bounds(lp.ndea, lower = temp.lb)  
-          
+          if(rts == "irs"){temp.lb[c(id.w.s1, id.w.s2)] <- -Inf; temp.ub[c(id.w.s1, id.w.s2)] <- 0}
+          set.bounds(lp.ndea, lower = temp.lb, upper = temp.ub)  
+
           # Solve
           solve.lpExtPtr(lp.ndea)
           
           # Get results
-          res.eff.cand <- c(res.eff.cand, abs(get.objective(lp.ndea)))
+          res.eff.cand <- c(res.eff.cand, ifelse(orientation == "i", abs(get.objective(lp.ndea)), 1/abs(get.objective(lp.ndea))))
         }
         
         # Pick the best system eff
@@ -193,18 +203,19 @@ function(xdata.s1, ydata.s1 = NULL, zdata, xdata.s2 = NULL, ydata.s2,
         dimnames(lp.ndea)[[2]] <- temp
         
         # Objective
-        if(orientation == "o") stop('DJL is too busy to implement OO model.')
+        if(orientation == "o") set.objfn(lp.ndea,  c(zdata[k,], if(is.null(xdata.s2)) NULL else xdata.s2[k,], 1), 
+                                         indices = c(id.p, if(is.null(xdata.s2)) NULL else id.v.s2, id.w.s2))
         if(orientation == "i") set.objfn(lp.ndea,  c(-ydata.s2[k,] * res.eff.s1[k,], 1 * res.eff.s1[k,]), 
                                          indices = c(id.u.s2, id.w.s2))
         
-        # RTS
+        # CRS
         if(rts == "crs") add.constraint(lp.ndea, c(1, 1), indices = c(id.w.s1, id.w.s2), "=", 0)
-        if(rts == "irs") stop('DJL is too busy to implement IRS model.')
-        if(rts == "drs") stop('DJL is too busy to implement DRS model.')
-        
+
         # Constraint for o
-        add.constraint(lp.ndea, c(zdata[k,], if(is.null(xdata.s2)) NULL else xdata.s2[k,]), 
-                       indices = c(id.p, if(is.null(xdata.s2)) NULL else id.v.s2), "=", 1)
+        if(orientation == "o") add.constraint(lp.ndea, c(ydata.s2[k,] * res.eff.s1[k,]), 
+                                              indices = c(id.u.s2), "=", 1)
+        if(orientation == "i") add.constraint(lp.ndea, c(zdata[k,], if(is.null(xdata.s2)) NULL else xdata.s2[k,]), 
+                                              indices = c(id.p, if(is.null(xdata.s2)) NULL else id.v.s2), "=", 1)  
         
         # Eff linking constraint
         add.constraint(lp.ndea, c(-xdata.s1[k,] * res.eff.s1[k,], 
@@ -230,27 +241,27 @@ function(xdata.s1, ydata.s1 = NULL, zdata, xdata.s2 = NULL, ydata.s2,
                                      id.u.s2, id.w.s2), "<=", 0)  
         }
         
-        # Bounds
-        temp.lb <- rep(0, no.dv.t)
+        # Bounds for VRS/IRS
+        temp.lb <- rep(  0, no.dv.t)
+        temp.ub <- rep(Inf, no.dv.t)
         if(rts == "vrs"){temp.lb[c(id.w.s1, id.w.s2)] <- -Inf}
-        set.bounds(lp.ndea, lower = temp.lb)  
+        if(rts == "irs"){temp.lb[c(id.w.s1, id.w.s2)] <- -Inf; temp.ub[c(id.w.s1, id.w.s2)] <- 0}
+        set.bounds(lp.ndea, lower = temp.lb, upper = temp.ub)  
         
         # Solve
         solve.lpExtPtr(lp.ndea)
         
         # Get results
-        res.all         <- get.variables(lp.ndea)
-        res.v.s1[k,]    <- res.all[id.v.s1]
-        res.u.s1[k,]    <- if(is.null(ydata.s1)) NA else res.all[id.u.s1]
-        res.p[k,]       <- res.all[id.p]
-        res.w.s1[k,]    <- res.all[id.w.s1]
-        res.v.s2[k,]    <- if(is.null(xdata.s2)) NA else res.all[id.v.s2]
-        res.u.s2[k,]    <- res.all[id.u.s2]
-        res.w.s2[k,]    <- res.all[id.w.s2]
-        res.eff.s2.temp <- sum(res.u.s2[k,] * ydata.s2[k,]) - res.w.s2[k,] - sum(c(res.p[k,], res.v.s2[k,]) * c(zdata[k,], xdata.s2[k,]))
-        res.eff.s2[k,]  <- ifelse(res.eff.s2.temp == 0, 1,
-                                  (sum(res.u.s2[k,] * ydata.s2[k,]) - res.w.s2[k,]) / (sum(c(res.p[k,], res.v.s2[k,]) * c(zdata[k,], xdata.s2[k,]))))
-
+        res.all        <- get.variables(lp.ndea)
+        res.v.s1[k,]   <- res.all[id.v.s1]
+        res.u.s1[k,]   <- if(is.null(ydata.s1)) NA else res.all[id.u.s1]
+        res.p[k,]      <- res.all[id.p]
+        res.w.s1[k,]   <- res.all[id.w.s1]
+        res.v.s2[k,]   <- if(is.null(xdata.s2)) NA else res.all[id.v.s2]
+        res.u.s2[k,]   <- res.all[id.u.s2]
+        res.w.s2[k,]   <- res.all[id.w.s2]
+        res.eff.s1[k,] <- ifelse(orientation == "i", res.eff.s1[k,], 1/res.eff.s1[k,])
+        res.eff.s2[k,] <- ifelse(orientation == "i", max(res.eff.cand) / res.eff.s1[k,], res.eff.s1[k,] / max(res.eff.cand))
       }
 
     }else{ # No external input/output
@@ -265,40 +276,31 @@ function(xdata.s1, ydata.s1 = NULL, zdata, xdata.s2 = NULL, ydata.s2,
         dimnames(lp.ndea)[[2]] <- temp
         
         # Objective
-        if(orientation == "o") stop('DJL is too busy to implement OO model.')
+        if(orientation == "o") set.objfn(lp.ndea, c(xdata.s1[k,], 1, 1), indices = c(id.v.s1, id.w.s1, id.w.s2))
         if(orientation == "i") set.objfn(lp.ndea, c(1, -ydata.s2[k,], 1), indices = c(id.w.s1, id.u.s2, id.w.s2))
         
-        # RTS
+        # CRS
         if(rts == "crs") add.constraint(lp.ndea, c(1, 1), indices = c(id.w.s1, id.w.s2), "=", 0)
-        if(rts == "irs") stop('DJL is too busy to implement IRS model.')
-        if(rts == "drs") stop('DJL is too busy to implement DRS model.')
-        
+
         # Constraint for o
-        add.constraint(lp.ndea, xdata.s1[k,], indices = id.v.s1, "=", 1)
+        if(orientation == "o") add.constraint(lp.ndea, ydata.s2[k,], indices = id.u.s2, "=", 1)
+        if(orientation == "i") add.constraint(lp.ndea, xdata.s1[k,], indices = id.v.s1, "=", 1)
         
         # Constraint for all
         for(d in o){
           # Stage 1
-          add.constraint(lp.ndea, c(-xdata.s1[d,], 
-                                    if(is.null(ydata.s1)) NULL else ydata.s1[d,],
-                                    zdata[d,], -1), 
-                         indices = c(id.v.s1, 
-                                     if(is.null(ydata.s1)) NULL else id.u.s1, 
-                                     id.p, id.w.s1), "<=", 0)
+          add.constraint(lp.ndea, c(-xdata.s1[d,], zdata[d,], -1), indices = c(id.v.s1, id.p, id.w.s1), "<=", 0)
           
           # Stage 2
-          add.constraint(lp.ndea, c(-zdata[d,],
-                                    if(is.null(xdata.s2)) NULL else -xdata.s2[d,],
-                                    ydata.s2[d,], -1), 
-                         indices = c(id.p, 
-                                     if(is.null(xdata.s2)) NULL else id.v.s2, 
-                                     id.u.s2, id.w.s2), "<=", 0)  
+          add.constraint(lp.ndea, c(-zdata[d,], ydata.s2[d,], -1), indices = c(id.p, id.u.s2, id.w.s2), "<=", 0)  
         }
         
-        # Bounds
-        temp.lb <- rep(0, no.dv.t)
+        # Bounds for VRS/IRS
+        temp.lb <- rep(  0, no.dv.t)
+        temp.ub <- rep(Inf, no.dv.t)
         if(rts == "vrs"){temp.lb[c(id.w.s1, id.w.s2)] <- -Inf}
-        set.bounds(lp.ndea, lower = temp.lb)  
+        if(rts == "irs"){temp.lb[c(id.w.s1, id.w.s2)] <- -Inf; temp.ub[c(id.w.s1, id.w.s2)] <- 0}
+        set.bounds(lp.ndea, lower = temp.lb, upper = temp.ub)  
         
         # Solve
         solve.lpExtPtr(lp.ndea)
@@ -306,16 +308,16 @@ function(xdata.s1, ydata.s1 = NULL, zdata, xdata.s2 = NULL, ydata.s2,
         # Get results
         res.all         <- get.variables(lp.ndea)
         res.v.s1[k,]    <- res.all[id.v.s1]
-        res.u.s1[k,]    <- if(is.null(ydata.s1)) NA else res.all[id.u.s1]
         res.p[k,]       <- res.all[id.p]
         res.w.s1[k,]    <- res.all[id.w.s1]
-        res.v.s2[k,]    <- if(is.null(xdata.s2)) NA else res.all[id.v.s2]
         res.u.s2[k,]    <- res.all[id.u.s2]
         res.w.s2[k,]    <- res.all[id.w.s2]
-        res.eff.s1[k,]  <- sum(c(res.u.s1[k,], res.p[k,]) * c(ydata.s1[k,], zdata[k,])) - res.w.s1[k,]
-        res.eff.s2.temp <- sum(res.u.s2[k,] * ydata.s2[k,]) - res.w.s2[k,] - sum(c(res.p[k,], res.v.s2[k,]) * c(zdata[k,], xdata.s2[k,]))
-        res.eff.s2[k,]  <- ifelse(res.eff.s2.temp == 0, 1,
-                                  (sum(res.u.s2[k,] * ydata.s2[k,]) - res.w.s2[k,]) / (sum(c(res.p[k,], res.v.s2[k,]) * c(zdata[k,], xdata.s2[k,]))))
+        res.eff.s1[k,]  <- ifelse(orientation == "i", 
+                                  sum(res.p[k,] * zdata[k,]) - res.w.s1[k,],
+                                  (sum(res.v.s1[k,] * xdata.s1[k,]) + res.w.s1[k,]) / sum(res.p[k,] * zdata[k,]))
+        res.eff.s2[k,]  <- ifelse(orientation == "i", 
+                                  (sum(res.u.s2[k,] * ydata.s2[k,]) - res.w.s2[k,]) / sum(res.p[k,] * zdata[k,]),
+                                  (sum(res.p[k,] * zdata[k,]) + res.w.s2[k,]) / sum(res.u.s2[k,] * ydata.s2[k,]))
       }
     }
 
@@ -325,6 +327,13 @@ function(xdata.s1, ydata.s1 = NULL, zdata, xdata.s2 = NULL, ydata.s2,
       # Leader
       res.eff.s1 <- dm.dea(xdata.s1, cbind(ydata.s1, zdata), rts, orientation)$eff
       
+      # Skip for zero-weighting inefficient DMU(s)
+      if(orientation == "i" & rts == "vrs"){
+        id.ineff  <- which(round(res.eff.s1, 8) != 1)
+        id.zero.u <- which(round(rowSums(dm.dea(xdata.s1, cbind(ydata.s1, zdata), rts, orientation)$u), 8) == 0)
+        o         <- setdiff(o, intersect(id.ineff, id.zero.u))
+      }
+      
       # Follower
       for(k in o){
         # Declare LP
@@ -337,21 +346,28 @@ function(xdata.s1, ydata.s1 = NULL, zdata, xdata.s2 = NULL, ydata.s2,
         dimnames(lp.ndea)[[2]] <- temp
         
         # Objective
-        if(orientation == "o") stop('DJL is too busy to implement OO model.')
+        if(orientation == "o") set.objfn(lp.ndea, c(if(is.null(xdata.s2)) NULL else xdata.s2[k,], zdata[k,], 1), 
+                                         indices = c(if(is.null(xdata.s2)) NULL else id.v.s2, id.p, id.w.s2))
         if(orientation == "i") set.objfn(lp.ndea, c(-ydata.s2[k,], 1), indices = c(id.u.s2, id.w.s2))
         
-        # RTS
+        # CRS
         if(rts == "crs") add.constraint(lp.ndea, c(1, 1), indices = c(id.w.s1, id.w.s2), "=", 0)
-        if(rts == "irs") stop('DJL is too busy to implement IRS model.')
-        if(rts == "drs") stop('DJL is too busy to implement DRS model.')
-        
+
         # Constraint for o
-        add.constraint(lp.ndea, c(-xdata.s1[k,] * res.eff.s1[k,], ydata.s1[k,], zdata[k,], -1), 
-                       indices = c(id.v.s1, if(is.null(ydata.s1)) NULL else id.u.s1, id.p, id.w.s1), "=", 0)
-        
-        add.constraint(lp.ndea, c(if(is.null(xdata.s2)) NULL else xdata.s2[k,], zdata[k,]), 
-                       indices = c(if(is.null(xdata.s2)) NULL else id.v.s2, id.p), "=", 1)
-        
+        if(orientation == "o"){
+          add.constraint(lp.ndea, c(-xdata.s1[k,] / res.eff.s1[k,], ydata.s1[k,], zdata[k,], -1/res.eff.s1[k,]), 
+                         indices = c(id.v.s1, if(is.null(ydata.s1)) NULL else id.u.s1, id.p, id.w.s1), "=", 0)
+          
+          add.constraint(lp.ndea, ydata.s2[k,], indices = id.u.s2, "=", 1)  
+        }
+        if(orientation == "i"){
+          add.constraint(lp.ndea, c(-xdata.s1[k,] * res.eff.s1[k,], ydata.s1[k,], zdata[k,], -1), 
+                         indices = c(id.v.s1, if(is.null(ydata.s1)) NULL else id.u.s1, id.p, id.w.s1), "=", 0)
+          
+          add.constraint(lp.ndea, c(if(is.null(xdata.s2)) NULL else xdata.s2[k,], zdata[k,]), 
+                         indices = c(if(is.null(xdata.s2)) NULL else id.v.s2, id.p), "=", 1)  
+        }
+
         # Constraint for all
         for(d in o){
           # Stage 1
@@ -369,20 +385,28 @@ function(xdata.s1, ydata.s1 = NULL, zdata, xdata.s2 = NULL, ydata.s2,
                          indices = c(id.p, 
                                      if(is.null(xdata.s2)) NULL else id.v.s2, 
                                      id.u.s2, id.w.s2), "<=", 0)  
-        }
-        
-        # Bounds
-        temp.lb <- rep(0, no.dv.t)
+        }  
+
+        # Bounds for VRS/IRS
+        temp.lb <- rep(  0, no.dv.t)
+        temp.ub <- rep(Inf, no.dv.t)
         if(rts == "vrs"){temp.lb[c(id.w.s1, id.w.s2)] <- -Inf}
-        set.bounds(lp.ndea, lower = temp.lb)  
+        if(rts == "irs"){temp.lb[c(id.w.s1, id.w.s2)] <- -Inf; temp.ub[c(id.w.s1, id.w.s2)] <- 0}
+        set.bounds(lp.ndea, lower = temp.lb, upper = temp.ub)  
         
         # Solve
         solve.lpExtPtr(lp.ndea)
         
         # Plan B for multiple optima
         if(solve.lpExtPtr(lp.ndea) == 3){
-          # Constraint for o: v1x1 = 1
-          add.constraint(lp.ndea, xdata.s1[k,], indices = id.v.s1, "=", 1)
+          if(orientation == "o"){
+            # Constraint for o: v1x1 + w1 = 1
+            add.constraint(lp.ndea, c(xdata.s1[k,], 1), indices = c(id.v.s1, id.w.s1), "=", 1)  
+          }
+          if(orientation == "i"){
+            # Constraint for o: v1x1 = 1
+            add.constraint(lp.ndea, xdata.s1[k,], indices = id.v.s1, "=", 1)  
+          }
 
           # Re-solve
           solve.lpExtPtr(lp.ndea)
@@ -397,13 +421,20 @@ function(xdata.s1, ydata.s1 = NULL, zdata, xdata.s2 = NULL, ydata.s2,
         res.v.s2[k,]   <- if(is.null(xdata.s2)) NA else res.all[id.v.s2]
         res.u.s2[k,]   <- res.all[id.u.s2]
         res.w.s2[k,]   <- res.all[id.w.s2]
-        res.eff.s2[k,] <- sum(res.u.s2[k,] * ydata.s2[k,]) - res.w.s2[k,]
+        res.eff.s2[k,] <- abs(get.objective(lp.ndea))
       }
       
     }else{
       # Leader
       res.eff.s2 <- dm.dea(cbind(xdata.s2, zdata), ydata.s2, rts, orientation)$eff
       
+      # Skip for zero-weighting inefficient DMU(s)
+      if(orientation == "o" & rts == "vrs"){
+        id.ineff  <- which(round(res.eff.s2, 8) != 1)
+        id.zero.v <- which(round(rowSums(dm.dea(cbind(xdata.s2, zdata), ydata.s2, rts, orientation)$v), 8) == 0)
+        o         <- setdiff(o, intersect(id.ineff, id.zero.v))
+      }
+      
       # Follower
       for(k in o){
         # Declare LP
@@ -416,26 +447,35 @@ function(xdata.s1, ydata.s1 = NULL, zdata, xdata.s2 = NULL, ydata.s2,
         dimnames(lp.ndea)[[2]] <- temp
         
         # Objective
-        if(orientation == "o") stop('DJL is too busy to implement OO model.')
+        if(orientation == "o") set.objfn(lp.ndea, c(xdata.s1[k,], 1), indices = c(id.v.s1, id.w.s1))
         if(orientation == "i") set.objfn(lp.ndea, c(-zdata[k,], 
                                                     if(is.null(ydata.s1)) NULL else -ydata.s1[k,], 1),
                                          indices = c(id.p, 
                                                      if(is.null(ydata.s1)) NULL else id.u.s1, id.w.s1))
         
-        # RTS
+        # CRS
         if(rts == "crs") add.constraint(lp.ndea, c(1, 1), indices = c(id.w.s1, id.w.s2), "=", 0)
-        if(rts == "irs") stop('DJL is too busy to implement IRS model.')
-        if(rts == "drs") stop('DJL is too busy to implement DRS model.')
-        
-        # Constraint for o
-        add.constraint(lp.ndea, c(-zdata[k,] * res.eff.s2[k,], 
-                                  if(is.null(xdata.s2)) NULL else -xdata.s2[k,] * res.eff.s2[k,], 
-                                  ydata.s2[k,], -1), 
-                       indices = c(id.p, 
-                                   if(is.null(xdata.s2)) NULL else id.v.s2, 
-                                   id.u.s2, id.w.s1), "=", 0)
 
-        add.constraint(lp.ndea, c(xdata.s1[k,]), indices = c(id.v.s1), "=", 1)
+        # Constraint for o
+        if(orientation == "o"){
+          add.constraint(lp.ndea, c(-zdata[k,] / res.eff.s2[k,], 
+                                    if(is.null(xdata.s2)) NULL else -xdata.s2[k,] / res.eff.s2[k,],
+                                    ydata.s2[k,], -1/res.eff.s2[k,]), 
+                         indices = c(id.p, if(is.null(xdata.s2)) NULL else id.v.s2, id.u.s2, id.w.s2), "=", 0)
+          
+          add.constraint(lp.ndea, c(zdata[k,], if(is.null(xdata.s2)) NULL else xdata.s2[k,]), 
+                         indices = c(id.p, if(is.null(xdata.s2)) NULL else id.v.s2), "=", 1)  
+        }
+        if(orientation == "i"){
+          add.constraint(lp.ndea, c(-zdata[k,] * res.eff.s2[k,], 
+                                    if(is.null(xdata.s2)) NULL else -xdata.s2[k,] * res.eff.s2[k,], 
+                                    ydata.s2[k,], -1), 
+                         indices = c(id.p, 
+                                     if(is.null(xdata.s2)) NULL else id.v.s2, 
+                                     id.u.s2, id.w.s1), "=", 0)
+          
+          add.constraint(lp.ndea, c(xdata.s1[k,]), indices = c(id.v.s1), "=", 1)
+        }
         
         # Constraint for all
         for(d in o){
@@ -456,18 +496,28 @@ function(xdata.s1, ydata.s1 = NULL, zdata, xdata.s2 = NULL, ydata.s2,
                                      id.u.s2, id.w.s2), "<=", 0)  
         }
         
-        # Bounds
-        temp.lb <- rep(0, no.dv.t)
+        # Bounds for VRS/IRS
+        temp.lb <- rep(  0, no.dv.t)
+        temp.ub <- rep(Inf, no.dv.t)
         if(rts == "vrs"){temp.lb[c(id.w.s1, id.w.s2)] <- -Inf}
-        set.bounds(lp.ndea, lower = temp.lb)  
+        if(rts == "irs"){temp.lb[c(id.w.s1, id.w.s2)] <- -Inf; temp.ub[c(id.w.s1, id.w.s2)] <- 0}
+        set.bounds(lp.ndea, lower = temp.lb, upper = temp.ub)  
         
         # Solve
         solve.lpExtPtr(lp.ndea)
         
         # Plan B for multiple optima
         if(solve.lpExtPtr(lp.ndea) == 3){
-          # Constraint for o: v1x1 = 1
-          add.constraint(lp.ndea, xdata.s1[k,], indices = id.v.s1, "=", 1)
+          if(orientation == "o"){
+            # Constraint for o: pz + (v2x2) + w2 = 1
+            add.constraint(lp.ndea, c(zdata[k,], if(is.null(xdata.s2)) NULL else xdata.s2[k,], 1), 
+                           indices = c(id.p, if(is.null(xdata.s2)) NULL else id.v.s2, id.w.s2), "=", 1)
+          }
+          if(orientation == "i"){
+            # Constraint for o: pz + (v2x2) = 1
+            add.constraint(lp.ndea, c(zdata[k,], if(is.null(xdata.s2)) NULL else xdata.s2[k,]), 
+                           indices = c(id.p, if(is.null(xdata.s2)) NULL else id.v.s2), "=", 1)
+          }
           
           # Re-solve
           solve.lpExtPtr(lp.ndea)
@@ -482,10 +532,9 @@ function(xdata.s1, ydata.s1 = NULL, zdata, xdata.s2 = NULL, ydata.s2,
         res.v.s2[k,]   <- if(is.null(xdata.s2)) NA else res.all[id.v.s2]
         res.u.s2[k,]   <- res.all[id.u.s2]
         res.w.s2[k,]   <- res.all[id.w.s2]
-        res.eff.s1[k,] <- sum(c(res.u.s1[k,], res.p[k,]) * c(ydata.s1[k,], zdata[k,])) - res.w.s1[k,]
+        res.eff.s1[k,] <- abs(get.objective(lp.ndea))
       }
     }
-    
   }
   
   # Returning object
